//	DialogBoxCloseButton.h - LookAndFeel subclass so that dialogs have the
//						     same close button as the main window (set in 
//						     NiallsPedalBoard.cpp).
//	--------------------------------------------------------------------------
//	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
//	Copyright (C) 2004-2005  Niall Moody
//	
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//	--------------------------------------------------------------------------
//	
//	You can e-mail me at niallmoody@yahoo.co.uk if you want.
//	
//	--------------------------------------------------------------------------

#ifndef DIALOGBOXCLOSEBUTTON_H_
#define DIALOGBOXCLOSEBUTTON_H_

#include <juce.h>

///	Simple LookAndFeel class to give our DialogWindows the same close button as the main window.
class DialogBoxCloseButton : public LookAndFeel
{
  public:
	///	Constructor.
	/*!
		\param image1 Normal image for the button.
		\param image2 Mouse-Over image for the button.
		\param image3 Depressed image for the button.
		\param resizerCol The colour used to draw the resizer bars.
	 */
	DialogBoxCloseButton(String image1,
						 String image2,
						 String image3,
						 Colour resizerColour);

	///	JUCE v1.12 introduced changes in how TextButtons are
	///	drawn, so we have to do this to go back to the original way they were
	///	drawn.
	/*!
		(as well as write our own TextButton subclass to draw coloured text).
	 */
	void drawButtonBackground(Graphics& g,
                              Button& button,
                              const Colour& backgroundColour,
                              bool isMouseOverButton,
                              bool isButtonDown);

	///	Creates a new close Button for DialogWindows.
	/*!
		This is how the LookAndFeel classes work - classes which make use of
		LookAndFeel will call a LookAndFeel object to determine how they
		should be drawn, what stages they go through at creation etc.

		Here we just replace the standard JUCE close button images with our
		own.
	 */
	Button *createDialogBoxCloseButton();

	///	Draws the resizer bars with our own colours.
	void drawCornerResizer(Graphics& g,
                                    int w, int h,
                                    bool isMouseOver,
                                    bool isMouseDragging);
  private:
	///	Path to the normal image for the button.
	String _image1;
	///	Path to the mouse-over image for the button.
	String _image2;
	///	Path to the depressed image for the button.
	String _image3;
	///	The colour used to draw the resizer bars.
	Colour _resizerColour;
};

///	Bit of a stupid way to do this, but I wanted to allow for the new shiny L&F.
class DBCButtonShiny : public ShinyLookAndFeel
{
  public:
	///	Constructor.
	/*!
		\param image1 Normal image for the button.
		\param image2 Mouse-Over image for the button.
		\param image3 Depressed image for the button.
		\param resizerCol The colour used to draw the resizer bars.
	 */
	DBCButtonShiny(String image1,
				   String image2,
				   String image3,
				   Colour resizerColour);

	///	JUCE v1.12 introduced changes in how TextButtons are
	///	drawn, so we have to do this to go back to the original way they were
	///	drawn.
	/*!
		(as well as write our own TextButton subclass to draw coloured text).
	 */
	/*void drawButtonBackground(Graphics& g,
                              Button& button,
                              const Colour& backgroundColour,
                              bool isMouseOverButton,
                              bool isButtonDown);*/

	///	Creates a new close Button for DialogWindows.
	/*!
		This is how the LookAndFeel classes work - classes which make use of
		LookAndFeel will call a LookAndFeel object to determine how they
		should be drawn, what stages they go through at creation etc.

		Here we just replace the standard JUCE close button images with our
		own.
	 */
	Button *createDialogBoxCloseButton();

	///	Draws the resizer bars with our own colours.
	void drawCornerResizer(Graphics& g,
                                    int w, int h,
                                    bool isMouseOver,
                                    bool isMouseDragging);
  private:
	///	Path to the normal image for the button.
	String _image1;
	///	Path to the mouse-over image for the button.
	String _image2;
	///	Path to the depressed image for the button.
	String _image3;
	///	The colour used to draw the resizer bars.
	Colour _resizerColour;
};

#endif
