//	NPBWindow.h - The main window, set up like a normal Windows
//				  window, with minimise, maximise and close
//				  buttons, and a resizer.
//	--------------------------------------------------------------------------
//	This file is part of 'Niall's Pedal Board', a simple ASIO VST host.
//	Copyright (C) 2004-2005  Niall Moody
//	
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//	
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//	
//	--------------------------------------------------------------------------
//	
//	You can e-mail me at niallmoody@yahoo.co.uk if you want.
//	
//	--------------------------------------------------------------------------

#ifndef NPBWINDOW_H_
#define NPBWINDOW_H_

#include "Joystick.h"
#include <juce.h>
#include "NiallsLabel.h"

class NPBWindow;
class ComponentFactory;

#ifndef NPBPLUGINORGANISER_H_
#include "NPBPluginOrganiser.h"
#endif

///	The main window for the program.
/*!
	Derived from Joystick to get joystick events, to pass on to the
	NPBPluginOrganiser, which then passes them on to each instantiated plugin
	in turn.
 */
class NPBWindow : public Component,
				  public ActionListener,
				  public Joystick
{
  public:
	///	Constructor.
	/*!
		\param xmlPath Path to the xml file used to create the GUI (i.e. the
		various components and their associated attributes.
		\param debug If true, the program will log a number of important
		actions to a 'NiallsPedalBoard.log' file.
	 */
	NPBWindow(String xmlPath, bool debug);
	///	Destructor.
	~NPBWindow();

	///	Called from the constructor, to build the gui from the xml file.
	void createGUI();

	///	Fills the background with a GradientBrush, draws the title text.
	void paint(Graphics& g);
	///	Used to handle the various buttons and the menus.
	void actionListenerCallback(const String& m);
	///	Called whenever a joystick event occurs.
	void joystickCallback(JoystickEvent& joystate, int index);

	///	Maximises the window.
	void maximise() {maxButtonPressed();};
	///	Minimises the window.
	void minimise() {minButtonPressed();};

	///	Positions the Components correctly.
	void resized();

	///	Used to determine when to move the window.
	/*!
		We have a section along the top of the window which acts as the
		window's title bar.  When you drag it, the window moves accordingly.
	 */
	void mouseDown(const MouseEvent& e);
	///	Used to determine when to move the window.
	/*!
		We have a section along the top of the window which acts as the
		window's title bar.  When you drag it, the window moves accordingly.
	 */
	void mouseDrag(const MouseEvent& e);
	///	Used to determine when to move the window.
	/*!
		We have a section along the top of the window which acts as the
		window's title bar.  When you drag it, the window moves accordingly.
	 */
	void mouseUp(const MouseEvent& e);

	///	Called from NPBOptionsDialog, to set the soundcard settings.
	void DialogOK(String soundcard, BitArray input, BitArray output);
	///	Not used(?).
	void DialogCancel();

	///	Sets whether joystick event values should be smoothed or not.
	/*!
		\sa ParameterFilter
	 */
	void setJoystickSmoothing(bool val) {JoystickSmoothing = val; setFilter(val);};
	///	Returns whether or not joystick event values are smoothed.
	/*!
		\sa ParameterFilter
	 */
	bool getJoystickSmoothing() {return JoystickSmoothing;};
	///	Sets the value of the envelope decrement for the envelope detector.
	void setEnvelopeDecrement(float val);
	///	Returns the value of the envelope decrement for the envelope detector.
	float getEnvelopeDecrement();

	///	Sets the current BPM the program passes onto interested plugins.
	void setBpm(float val);
	///	Returns the current BPM the program is using.
	float getBpm();

	///	Returns the name of the current soundcard.
	String getSoundcard() {return Soundcard;};
	///	Returns which soundcard inputs are being used.
	BitArray getInputs() {return Inputs;};
	///	Returns which soundcard outputs are being used.
	BitArray getOutputs() {return Inputs;};
	///	Returns whether or not we're using direct mode for the audio.
	bool getDirectMode();
	///	Returns the default path to the folder which contains plugin patches/configurations.
	/*!
		The default path is the last path used to load/save plugin
		patches/configurations.  It is saved in the NPBProperties.prp file.
	 */
	String getPatchPath() {return PatchPath;};
	///	Sets the default path to the folder which contains plugin patches/configurations.
	/*!
		The default path is the last path used to load/save plugin
		patches/configurations.  It is saved in the NPBProperties.prp file.
	 */
	void setPatchPath(String& path) {PatchPath = path;};
	///	Returns the last used path to plugin dlls.
	/*!
		The default path is the last path used to load a plugin.  It is saved
		in the NPBProperties.prp file.
	 */
	String getPluginPath() {return PluginPath;};
	///	Sets the last used path to plugin dlls.
	/*!
		The default path is the last path used to load a plugin.  It is saved
		in the NPBProperties.prp file.
	 */
	void setPluginPath(String& path) {PluginPath = path;};
	///	Sets whether or not we're using direct mode for the audio.
	void setDirectMode(bool val);

	///	Returns an array of the names of the joysticks attached to the system.
	StringArray getJoysticks() {return Joysticks;};

	///	Method for the MainWindow FactoryObject to set whether the window's sizer is displayed.
	void setResizer(bool val);
	///	Method for the MainWindow FactoryObject to set whether the window may be maximised.
	void setMaximise(bool val);
	///	Method for the MainWindow FactoryObject to set the background colour of the window.
	void setColourRed(int val);
	///	Method for the MainWindow FactoryObject to set the background colour of the window.
	void setColourGreen(int val);
	///	Method for the MainWindow FactoryObject to set the background colour of the window.
	void setColourBlue(int val);
	///	Method for the MainWindow FactoryObject to set whether the drop shadow is displayed.
	void setDropShadow(bool val);
	///	Method for the MainWindow FactoryObject to set the title text's colour.
	void setTitleColourRed(int val);
	///	Method for the MainWindow FactoryObject to set the title text's colour.
	void setTitleColourGreen(int val);
	///	Method for the MainWindow FactoryObject to set the title text's colour.
	void setTitleColourBlue(int val);

	///	Called by the ComponentFactory to set the texts for the AudioButton.
	void setAudioButtonText(String val1, String val2);
	///	Called by the ComponentFactory to set the texts for the RecordButton.
	void setRecordButtonText(String val1, String val2);

	///	Returns the background colour used for the main window (this is used for dialog windows as well).
	Colour getBackgroundColour() {return colour;};

	///	Sets the x position of the title text.
	void setTitleX(int val) {titleX = val;};
	///	Sets the Y position of the title text.
	void setTitleY(int val) {titleY = val;};

	///	To set the lookAndFeel member.
	void setLAndF(int val) {lookAndFeel = val;};
	///	Returns the lookAndFeel member, so we can set it in the skinning methods.
	int getLAndF() {return lookAndFeel;};

	///	Sets the MIDI input device to be used by the plugins.
	void setMIDIInputDevice(int index);
	///	Returns the currently selected MIDI input device.
	int getMIDIInputDevice();
  private:
	///	This file is used to store various settings for the program.
	/*!
		Settings stored are:
		- Joystick smoothing.
		- Soundcard.
		- Soundcard inputs.
		- Soundcard outputs.
		- MIDI input device.
		- Patch/configuration path.
		- Plugin path.
	 */
	PropertiesFile *Prop;

	///	Called when the window's close button is pressed.
	void closeButtonPressed();
	///	Called when the window's minimise button is pressed.
	void minButtonPressed();
	///	Called when the window's maximise button is pressed.
	void maxButtonPressed();
	///	Called when the Audio Start/Stop button is pressed.
	void AudioButtonPressed();
	///	Called when the Audio Options button is pressed.
	void OptionsButtonPressed();
	///	Called when the Record button is pressed.
	void RecordButtonPressed();

	///	We use this to write all the soundcard options available on the system
	///	to the log file.
	void debugWriteSoundcardOptions();

	ImageButton *closeButton;
	ImageButton *minButton;
	ImageButton *maxButton;

	NiallsLabel *FileMenu;
	NiallsLabel *OptionsMenu;
	NiallsLabel *HelpMenu;

	TextButton *AudioButton;
	TextButton *OptionsButton;
	TextButton *RecordButton;

	ResizableCornerComponent *resizer;

	NPBPluginOrganiser *Organiser;

	///	Used when dragging the title bar, so that the window doesn't move to the mouse's exact position.
	int MouseOffset_x;
	///	Used when dragging the title bar, so that the window doesn't move to the mouse's exact position.
	int MouseOffset_y;

	bool Maximised;
	bool MoveWindow;
	///	Still used?
	bool OptionsOpen;

	Rectangle oldSize;

	String Soundcard;
	BitArray Inputs;
	BitArray Outputs;

	///	Doesn't work :(
	Typeface *SubvertAudioFont;

	StringArray Joysticks;
	bool JoystickSmoothing;
	String PatchPath;
	String PluginPath;

	///	The factory we use to construct the gui from the corresponding xml file.
	ComponentFactory *factory;

	///	The main colour for the background.
	Colour colour;
	///	The colour for the window's title text.
	Colour titleColour;
	///	The text displayed by the AudioButton when the audio is stopped.
	String abText1;
	///	The text displayed by the AudioButton when the audio is running.
	String abText2;
	///	The text displayed by the RecordButton when it's not recording.
	String rbText1;
	///	The text displayed by the RecordButton when it is recording.
	String rbText2;

	///	The x position of the title text.
	int titleX;
	///	The y position of the title text.
	int titleY;

	///	We use this to let skinners choose to use the default L&F, or the new shiny one.
	int lookAndFeel;
};

#endif
